import React, { useEffect, useState } from 'react';
import { View, ToastAndroid, AppState } from 'react-native';
// material
import { Text } from 'react-native-paper';
import { createMaterialTopTabNavigator } from '@react-navigation/material-top-tabs';
import MaterialCommunityIcons from 'react-native-vector-icons/MaterialCommunityIcons';
// redux
import { useDispatch } from 'react-redux';
import { useDeviceList, useDeviceActive, useReports, useSelectedReport } from './provider/device.helper';
// core
import { Storage } from './core/storage';
import { useDevicesAsString, useSMSData, canUseSMS } from './core/interpreter';
import { Shared } from './core/share';
import { useSMSRead } from './core/sms.read';
import { useSMSListener } from './core/sms.listen';
// component
import { AppBar, Selection } from './components';
// database
import { Slides } from './database/slides';
import { useNavigation } from '@react-navigation/core';

const Tab = createMaterialTopTabNavigator();

export const App = () => {
	const navigation = useNavigation();
	const dispatch = useDispatch();

	const [devices, setDevices] = useDeviceList(dispatch);
	const [deviceActive, setDeviceActive] = useDeviceActive(dispatch);
	const [, setReports] = useReports(dispatch);
	const [selectedReport, setSelectedReport] = useSelectedReport(dispatch);

	const [loading, setLoading] = useState(true);
	const [inited, setInited] = useState(false);

	useEffect(() => {
		console.log('==================================================');
		const subscription = AppState.addEventListener('change', (appState) => {
			console.log('App State = ' + appState);
			// on app reopen
			if (appState === 'active') {
				onAppStateActive();
			}
		});

		return () => {
			subscription.remove();
		};
	}, []);

	useEffect(() => {
		if (inited === false) {
			console.log('--------------------------------------------------');
			onInit();
		}
		// eslint-disable-next-line react-hooks/exhaustive-deps
	}, []);

	useEffect(() => {
		console.log('Loading Status', loading);
	}, [loading]);

	// on active device change => read reports and set report and set last report
	useEffect(() => {
		if (inited === false) {
			return;
		}
		console.log('App.js/useEffect/deviceActive');
		onActiveDeviceChange();
		// eslint-disable-next-line react-hooks/exhaustive-deps
	}, [deviceActive]);

	const onInit = async () => {
		try {
			// load data from storage
			const [_deviceList, _deviceActive] = await Promise.all([
				Storage.ListDevice(),
				Storage.GetDeviceActive(),
			]);
			setInited(true);
			// set devices or java via shared
			const _str_devices = useDevicesAsString(_deviceList);
			Shared.SendDevices(_str_devices);
			// set devices to redux
			setDevices(_deviceList);
			// set device active to redux
			setDeviceActive(_deviceActive);
			setTimeout(() => {
				console.log(`Selected device is : ${deviceActive}/${_deviceActive}`);
				// start forground service
				onForground();
			}, 0);
			if (_deviceList.length === 0) {
				ToastAndroid.show('شما هیچ دستگاه ثبت شده ندارید.', 3000);
				setTimeout(() => {
					navigation.navigate('settings');
				}, 300);
			}
		} catch (error) {
			console.error('App.js/onInit');
			console.error(error);
			setLoading(false);
		}
	};

	const onForground = async () => {
		try {
			console.log('forground/start');
			useSMSListener((message) => {
				setTimeout(async () => {
					try {
						// load data from storage
						const [_deviceList, _deviceActive] = await Promise.all([
							Storage.ListDevice(),
							Storage.GetDeviceActive(),
						]);
						console.log('forground/recived'); // if active device availabe fetch data fro sms
						console.log(_deviceActive);
						if (_deviceActive !== -1) {
							console.log('We have an active device');
							// if sent message was for active device
							if (message.originatingAddress === _deviceList[_deviceActive].phone) {
								console.log('We have a new report');
								let _reports = [];
								const SMSs = await useSMSRead({ phone: '' });
								for (let i = 0; i < SMSs.list.length; i++) {
									const text = SMSs.list[i].body;
									if (
										canUseSMS(text) &&
										SMSs.list[i].address === _deviceList[_deviceActive].phone
									) {
										let _data = useSMSData(text);
										if (_data === undefined) continue;
										if (_data && _data.date && _data.date.length !== 0) {
											_reports.push(_data);
										} // convert each string sms to readable data
									}
								}
								setReports(_reports);
								if (selectedReport !== -1) {
									// inc selected report
									let _selectedReport = selectedReport;
									console.log('prev selected report/' + _selectedReport);
									_selectedReport += 1;
									setSelectedReport(_selectedReport);
									console.log('new selected report/' + _selectedReport);
								} else {
									setSelectedReport(0);
								}
								ToastAndroid.show('گزارش جدید دریافت و اضافه شد.', 3000);
							}
						}

						setLoading(false);
					} catch (error) {
						setLoading(false);
						console.error('App.js/onForground');
						console.error(error);
					}
				}, 0);
			});
		} catch (error) {
			console.error('App.js/onForground');
			console.error(error);
		}
	};

	const onActiveDeviceChange = async () => {
		try {
			console.log('App.js/onActiveDeviceChange');
			const [_deviceList, _selectedDevice] = await Promise.all([
				Storage.ListDevice(),
				Storage.GetDeviceActive(),
			]);

			// if active device availabe fetch data fro sms
			if (_selectedDevice !== -1 && _deviceList.length !== 0) {
				console.log('Loading True at onActiveDeviceChange');
				setLoading(true);
				setReports([]);
				setSelectedReport(-1);
				// read SMS
				const phones = _deviceList.map((item) => item.phone); // list of device's phone number
				const SMSs = await useSMSRead({ phone: '' }); // if there were report SMSs

				let selectedPhone = _selectedDevice != -1 ? phones[_selectedDevice] : undefined; // selected device's phone number
				if (SMSs.count !== 0) {
					let _reports = [];
					for (let i = 0; i < SMSs.list.length; i++) {
						const text = SMSs.list[i].body;
						const phone = SMSs.list[i].address;

						if (canUseSMS(text) && canUsePhone(phones, phone, selectedPhone)) {
							let _data = useSMSData(text);
							console.log(_data);
							if (_data === undefined) continue;
							if (_data && _data.date && _data.date.length !== 0) {
								_reports.push(_data);
							} // convert each string sms to readable data
						}
					}
					setReports(_reports);
					setSelectedReport(0); // select last report as selected report
					ToastAndroid.show('گزارشات با موفقیت خوانده شدند.', 3000);
				}
				setLoading(false);
			}
		} catch (error) {
			setLoading(false);
			console.error('App.js/onActiveDeviceChange');
			console.error(error);
		}
	};

	const onAppStateActive = async () => {
		try {
			console.log('App.js/onAppSteteActive');
			setLoading(true);

			// load data from storage
			const [_deviceList, _selectedDevice] = await Promise.all([
				Storage.ListDevice(),
				Storage.GetDeviceActive(),
			]);

			// if user saved a device
			if (_deviceList.length !== 0) {
				const phones = _deviceList.map((item) => item.phone); // list of device's phone number
				let selectedPhone = _selectedDevice != -1 ? phones[_selectedDevice] : undefined; // selected device's phone number
				// read SMS
				const SMSs = await useSMSRead({ phone: '' }); // if there were report SMSs

				if (SMSs.count !== 0) {
					let _reports = [];
					for (let i = 0; i < SMSs.list.length; i++) {
						const text = SMSs.list[i].body;
						const phone = SMSs.list[i].address;

						if (canUseSMS(text) && canUsePhone(phones, phone, selectedPhone)) {
							console.log('App.js/onAppStateActive - selectedPhone/' + phone);
							let _data = useSMSData(text);
							console.log(_data);
							if (_data === undefined) continue;
							if (_data && _data.date && _data.date.length !== 0) {
								_reports.push(_data);
							} // convert each string sms to readable data
							if (selectedPhone === undefined) {
								selectedPhone = phone;
							}
						}
					}
					setReports(_reports);
					setSelectedReport(0); // select last report as selected report
					ToastAndroid.show('گزارشات با موفقیت خوانده شدند.', 3000);
				}
			}

			setLoading(false);
		} catch (error) {
			setLoading(false);
			console.error('App.js/onActiveDevuceChange');
			console.error(error);
		}
	};

	const onRefresh = async () => {
		try {
			console.log('App.js/onRefresh');
			setLoading(true);

			// load data from storage
			const [_deviceList, _selectedDevice] = await Promise.all([
				Storage.ListDevice(),
				Storage.GetDeviceActive(),
			]);

			// if user saved a device
			if (_deviceList.length !== 0) {
				const phones = _deviceList.map((item) => item.phone); // list of device's phone number
				let selectedPhone = _selectedDevice != -1 ? phones[_selectedDevice] : undefined; // selected device's phone number
				// read SMS
				const SMSs = await useSMSRead({ phone: '' }); // if there were report SMSs

				if (SMSs.count !== 0) {
					let _reports = [];
					for (let i = 0; i < SMSs.list.length; i++) {
						const text = SMSs.list[i].body;
						const phone = SMSs.list[i].address;

						if (canUseSMS(text) && canUsePhone(phones, phone, selectedPhone)) {
							console.log('App.js/onAppStateActive - selectedPhone/' + phone);
							let _data = useSMSData(text);
							if (_data === undefined) continue;
							if (_data && _data.date && _data.date.length !== 0) {
								_reports.push(_data);
							} // convert each string sms to readable data
							if (selectedPhone === undefined) {
								selectedPhone = phone;
							}
						}
					}
					setReports(_reports);
					ToastAndroid.show('گزارشات با موفقیت خوانده شدند.', 3000);
					setLoading(false);
				}
			} else {
				setLoading(false);
			}
		} catch (error) {
			setLoading(false);
			console.error('App.js/onRefresh');
			console.error(error);
		}
	};

	/**
	 *
	 * @param {string[]} phones
	 * @param {string} phone
	 * @param {string} selectedPhone
	 * @returns {boolean}
	 */
	const canUsePhone = (phones, phone, selectedPhone) => {
		return (
			(phone.startsWith('0') && canUsePhone(phones, '+98' + phone.slice(1, phone.length)),
			selectedPhone) ||
			(selectedPhone &&
				selectedPhone.startsWith('0') &&
				canUsePhone(
					phones,
					phone,
					'+98' + selectedPhone.slice(1, selectedPhone.length)
				)(phones.includes(phone) && (selectedPhone === undefined || selectedPhone === phone)))
		);
	};

	return (
		<>
			<AppBar loading={loading} onRefresh={onRefresh} />
			<Selection />
			<Tab.Navigator
				initialRouteName="info"
				style={{
					direction: 'rtl',
				}}>
				{Slides.map((item, index) => (
					<Tab.Screen
						key={index}
						name={item.name}
						component={item.component ?? Test}
						options={{
							tabBarStyle: {
								backgroundColor: '#fff',
								borderBottomWidth: 1,
								borderBottomColor: 'rgba(0, 0, 0, 0.2)',
							},
							tabBarItemStyle: { width: 100 },
							tabBarScrollEnabled: true,
							tabBarLabel: () => (
								<>
									<View
										style={{
											flexDirection: 'row',
											alignItems: 'center',
											justifyContent: 'center',
										}}>
										<MaterialCommunityIcons name={item.icon} size={16} color="#000" />
										<Text style={{ marginLeft: 12 }}>{item.label}</Text>
									</View>
								</>
							),
						}}
					/>
				))}
			</Tab.Navigator>
		</>
	);
};
