import React, { useEffect, useState } from 'react';
import { View, ScrollView, Dimensions } from 'react-native';
// material
import { Text, Checkbox, Button } from 'react-native-paper';
import { Request } from '../../core/request';
// database
import { Alarms } from '../../database/alarm';
import { DialogConfirm } from '../../dialog/confirm';
// redux
import { useDispatch } from 'react-redux';
import { useDeviceActive, useDeviceList } from '../../provider/device.helper';

const { width } = Dimensions.get('window');

export const Alarm = () => {
	const dispatch = useDispatch();

	const [devices] = useDeviceList(dispatch);
	const [activeDevice] = useDeviceActive(dispatch);

	const [checks, setChecks] = useState(Array.from({ length: Alarms.length }, () => false));
	const [allChecks, setAllChecks] = useState(false);
	const [action, setAction] = useState(''); // actions like: alarm off/on
	const [showConfirmDialog, setShowConfirmDialog] = useState(false);

	useEffect(() => {
		setAllChecks(toggleChecksState());
		// eslint-disable-next-line react-hooks/exhaustive-deps
	}, []);

	const allArrayEqual = (arr) => arr.every((v) => v === arr[0]);

	const toggleChecksState = () => {
		return allArrayEqual(checks) && checks[0] === true;
	};

	const toggleChecks = () => {
		// if all were unchecked
		if (!allChecks) {
			setChecks(Array.from({ length: Alarms.length }, () => true));
			setAllChecks(true);
		}
		// if all were checked
		if (allChecks) {
			setChecks(Array.from({ length: Alarms.length }, () => false));
			setAllChecks(false);
		}
	};

	const toggleCheck = (index) => {
		let _checks = [...checks];

		_checks[index] = !_checks[index];

		setChecks(_checks);
		// check for all
		setAllChecks(!toggleChecksState());
	};

	const reverse = () => {
		let _checks = [...checks];

		for (let i in _checks) {
			_checks[i] = !_checks[i];
		}

		setChecks(_checks);
		// check for all
		setAllChecks(!toggleChecksState());
	};

	const calcAlarmBit = () => {
		let bit = 0;

		for (let i = 0; i < checks.length; i++) {
			if (checks[i] === true) {
				const oi = i + 1;
				bit += Math.pow(2, oi);
			}
		}

		return bit.toString();
	};

	const onConfirmDialogClose = (status) => {
		setShowConfirmDialog(false);

		if (status && devices[activeDevice]) {
			try {
				switch (action) {
					case 'ALARM/OFF':
						Request.SendAlarmOff(devices[activeDevice].phone);
						break;

					case 'ALARM/ON':
						Request.SendAlarmOn(devices[activeDevice].phone);
						break;

					case 'ALARM/SET':
						const bit = calcAlarmBit();
						Request.SendAlarmSet(devices[activeDevice].phone, bit);
						break;

					default:
						break;
				}
			} catch (error) {}

			setAction('');
		}
	};

	const openConfirmDialog = (value) => {
		setAction(value);
		setShowConfirmDialog(true);
	};

	const Header = () => (
		<View
			style={{
				padding: 10,
				backgroundColor: '#fff',
				borderBottomWidth: 1,
				borderBottomColor: 'rgba(0, 0, 0, 0.12)',
				flexDirection: 'row',
				alignItems: 'center',
			}}>
			<Checkbox status={allChecks ? 'checked' : 'unchecked'} onPress={toggleChecks} />
			<Text>انتخاب {allChecks ? 'هیچ کدام' : 'همه'}</Text>
			<View style={{ flex: 1 }} />
			<Button icon="autorenew" onPress={reverse}>
				معکوس
			</Button>
		</View>
	);

	const Footer = () => (
		<View
			style={{
				padding: 10,
				backgroundColor: '#fff',
				borderTopWidth: 1,
				borderTopColor: 'rgba(0, 0, 0, 0.12)',
				flexDirection: 'row',
			}}>
			<Button onPress={() => openConfirmDialog('ALARM/OFF')} style={{ flex: 1 }}>
				خاموش کلی
			</Button>
			<Button onPress={() => openConfirmDialog('ALARM/ON')} style={{ flex: 1 }}>
				روشن کلی
			</Button>
			<Button onPress={() => openConfirmDialog('ALARM/SET')} mode="contained" style={{ flex: 1 }}>
				ارسال
			</Button>
		</View>
	);

	return (
		<>
			<View style={{ flex: 1 }}>
				<Header />
				<ScrollView style={{ flex: 1 }}>
					<View style={{ flex: 1, flexWrap: 'wrap', flexDirection: 'row' }}>
						{Alarms.map((item, index) => (
							<View
								key={index}
								style={{
									width: width / 3,
									flexDirection: 'row',
									alignItems: 'center',
									paddingHorizontal: 16,
									paddingVertical: 8,
									borderTopWidth: 1,
									borderTopColor: 'rgba(0, 0, 0, 0.12)',
								}}>
								<Checkbox
									status={checks[index] ? 'checked' : 'unchecked'}
									onPress={() => toggleCheck(index)}
								/>
								<Text>{item}</Text>
							</View>
						))}
					</View>
				</ScrollView>
				<Footer />
			</View>

			<DialogConfirm
				visible={showConfirmDialog}
				onClose={onConfirmDialogClose}
				data={{
					title: 'ارسال آلارم',
					subtitle: 'آیا اطمینان برای ارسال این آلارم از طریق پیامک را دارید ؟',
				}}
			/>
		</>
	);
};
