import React, { useState } from 'react';
import { View, ScrollView } from 'react-native';
// material
import { Button, AnimatedFAB, List, IconButton, Text } from 'react-native-paper';
// dialog
import { DialogDeviceForm } from '../../dialog/deviceform';
import { DialogConfirm } from '../../dialog/confirm';
import { DialogRedata } from '../../dialog/refdata';
// redux
import { useDispatch } from 'react-redux';
import { useDeviceList, useDeviceActive } from '../../provider/device.helper';
// core
import { Storage } from '../../core/storage';
import { useDevicesAsString } from '../../core/interpreter';
import { Shared } from '../../core/share';
import { Request } from '../../core/request';
import { usePhoneCall } from '../../core/call';

export const Settings = () => {
	const dispath = useDispatch();

	const [devices, setDevices] = useDeviceList(dispath);
	const [deviceActive, setDeviceActive] = useDeviceActive(dispath);

	const [selected, setSelected] = useState(-1);
	const [showDeviceDialog, setShowDeviceDialog] = useState(false);
	const [showConfirmDialog, setShowConfirmDialog] = useState(false);
	const [showResetIPConfirmDialog, setShowResetIPConfirmDialog] = useState(false);
	const [showResetDataConfirmDialog, setShowResetDataConfirmDialog] = useState(false);

	const onDeviceDialogClose = (data) => {
		setShowDeviceDialog(false);

		// if data exist and user don't want to cancel dialog
		if (data) {
			const _devices = [...devices];

			let { name, phone, prefix } = data;

			phone = phone.replace(/\s/g, '');

			if (phone.slice(0, 1) === '0' && prefix == true) {
				phone = '+98' + phone.slice(1, phone.length);
			}

			if (selected === -1) {
				// wants to add a device
				_devices.push({ name, phone });
			} else {
				// wants to edit a device
				_devices[selected] = { name, phone };
			}
			// save to state
			setDevices(_devices);
			// save to storage
			Storage.SetDevice(_devices);
			// save to java with shared
			const _str_devices = useDevicesAsString(_devices);
			Shared.SendDevices(_str_devices);

			// set device to active device
			if (deviceActive === -1 || _devices.length === 1) {
				setDeviceActive(0);
				Storage.SetDeviceActive(0);
			}
		}

		// reset selected
		setSelected(-1);
	};

	const onConfirmDialogClose = (result) => {
		setShowConfirmDialog(false);

		// user want's to delete a device
		if (result === true) {
			// duplicate to proccess on current state
			const _devices = [...devices];
			// remove selected device
			_devices.splice(selected, 1);

			// set device to active device
			if (deviceActive === selected) {
				let index = -1; // set active -1 if device list is empty
				if (_devices.length !== 0) {
					// if devices list was not empty
					index = _devices.length; // select latest
				}

				setDeviceActive(index);
				Storage.SetDeviceActive(index);
			}

			// save to state
			setDevices(_devices);
			// save to storage
			Storage.SetDevice(_devices);
			// save to java with shared
			const _str_devices = useDevicesAsString(_devices);
			Shared.SendDevices(_str_devices);
		}

		// reset selected
		setSelected(-1);
	};

	const onResetIPConfirmDialogClose = (result) => {
		setShowResetIPConfirmDialog(false);

		if (result && devices[deviceActive]) {
			Request.SendResetIP(devices[deviceActive].phone);
		}
	};

	const onResetDataConfirmDialogClose = (result) => {
		setShowResetDataConfirmDialog(false);

		if (result !== null && devices[deviceActive]) {
			if (result === false) {
				usePhoneCall(devices[deviceActive].phone);
			} else {
				// send sms
			}
		}
	};

	const selectDeviceForEdit = (index) => {
		setSelected(index);
		setShowDeviceDialog(true);
	};

	const selectDeviceForDelete = (index) => {
		setSelected(index);
		setShowConfirmDialog(true);
	};

	const Header = () => (
		<View
			style={{
				padding: 10,
				backgroundColor: '#fff',
				borderBottomWidth: 1,
				borderBottomColor: 'rgba(0, 0, 0, 0.12)',
				flexDirection: 'row',
				alignItems: 'center',
			}}>
			<Button onPress={() => setShowResetDataConfirmDialog(true)} style={{ flex: 1 }} icon="reload">
				دریافت مجدد اطلاعات
			</Button>

			<Button
				onPress={() => setShowResetIPConfirmDialog(true)}
				style={{ flex: 1 }}
				icon="format-rotate-90">
				ریست کردن آیپی
			</Button>
		</View>
	);

	/**
	 *
	 * @param {{index: number, name: string, phone: string}} props
	 * @returns {JSX.Element}
	 */
	const Item = ({ index, name, phone }) => {
		return (
			<List.Item
				style={{
					borderBottomWidth: 1,
					borderBottomColor: 'rgba(0, 0, 0, 0.12)',
				}}
				title={name}
				right={() => (
					<>
						<IconButton icon="pencil" onPress={() => selectDeviceForEdit(index)} />
						<IconButton icon="delete" onPress={() => selectDeviceForDelete(index)} />
					</>
				)}
			/>
		);
	};

	return (
		<>
			<View style={{ flex: 1 }}>
				<Header />

				<ScrollView style={{ flex: 1 }}>
					<List.Section>
						{deviceActive !== -1 && devices[deviceActive] && (
							<>
								<List.Subheader>دستگاه انتخاب شده</List.Subheader>
								<Item
									index={deviceActive}
									name={devices[deviceActive].name}
									phone={devices[deviceActive].phone}
								/>
							</>
						)}
						<List.Subheader>دستگاه های ثبت شده</List.Subheader>
						{devices.map((item, index) => (
							<Item key={index} index={index} name={item.name} phone={item.phone} />
						))}
					</List.Section>
					{devices.length === 0 && (
						<Text style={{ marginHorizontal: 20 }}>
							شما هیچ دستگاه ثبت شده ای ندارید، لطفا یک دستگاه ثبت کنید.
						</Text>
					)}

					<View style={{ flex: 1, height: 120 }} />
				</ScrollView>

				<AnimatedFAB
					style={{ position: 'absolute', bottom: 20, right: 20 }}
					extended={true}
					icon="plus"
					label="افزودن دستگاه جدید"
					onPress={() => selectDeviceForEdit(-1)}
				/>
			</View>

			<DialogDeviceForm
				visible={showDeviceDialog}
				onClose={onDeviceDialogClose}
				data={selected !== -1 ? devices[selected] : null}
			/>

			<DialogConfirm
				visible={showConfirmDialog}
				onClose={onConfirmDialogClose}
				data={{
					title: 'حذف دستگاه',
					subtitle: 'آیا می خواهید این دستگاه را حذف کنید ؟',
				}}
			/>

			<DialogConfirm
				visible={showResetIPConfirmDialog}
				onClose={onResetIPConfirmDialogClose}
				data={{
					title: 'ریست آیپی',
					subtitle: 'آیا می خواهید ایپی ها را ریست کنید ؟',
				}}
			/>

			<DialogRedata visible={showResetDataConfirmDialog} onClose={onResetDataConfirmDialogClose} />
		</>
	);
};
